"""main module include `Issues` class to handle jira requests and print data in stdout.
"""

import os
import sys
import json

import requests

from jirakosaar.issues.auth.auth import BasicAuthentication


BASE_URL = "https://uxcamcom.atlassian.net/rest/api/3"


def parse_issues(issues: list) -> list:
    """Get key, summary and labels which is created as a statement.
    Output Sample
    -------------
        [
            {
                "is_completed": True,
                "statement": "ENG-1 : Create sprint ticket [backend]"
            }
        ]
    """

    data = []
    for item in issues:
        # TODO add tag newly_created automatically matching sprint started date and issue date
        # eg. created_date = i.get('fields').get('created') # to get [newly created] tag
        labels = ' '.join(f"[{item}]" for item in item.get('fields').get('labels'))

        def is_completed():
            if not item.get('fields').get('status').get('name') == 'Done':
                return False
            return True

        data.append({
            "is_completed": is_completed(),
            "statement": f"{item.get('key')} : {item.get('fields').get('summary')} {labels}"
        })
    return data


class Issues:
    """Search config, requets in the api endpoint and parse response. Reponse is then stdout
    in the terminal.
    Usage Example
    ----------
        >>> from jirakosaar.issues.main import Issues

        >>> issues = Issues("path/to/config")
        >>> issues.get_backlog_issues()
    """
    def __init__(self, config_dir: str):
        self.config_dir = config_dir
        self.config_file = os.path.join(self.config_dir, "config.json")

    def __get_auth_config(self) -> dict:
        try:
            with open(self.config_file, "r") as file:
                data = json.loads(file.read())
                return data
        except FileNotFoundError:
            print(f"Configuration file not found!\nNeed {self.config_file}")
            sys.exit()
        sys.exit()

    def get_response(self, jql: str) -> dict:
        """Create request endpoint with necessary headers and params, then send
        request and return response as a `dict`.
        Example
        --------
            >>> jql = 'resolution = "Done"'
            >>> get_response(jql)
        """

        basic_auth = BasicAuthentication(self.__get_auth_config())
        data = {
            "headers": {"Accept": "application/json"},
            "auth": basic_auth(),
            "params": {'jql': jql}
        }

        response = requests.request(
            "GET",
            f"{BASE_URL}/search",
            **data
        )

        if response.status_code != 200:
            sys.exit("\nI could not get 200 success status! \
                \nPlease check your auth credential or internet.")

        return json.loads(response.text).get('issues')

    def get_current_issues(self):
        """Stdout issues of authenticated user of open sprint.
        """

        jql = 'Sprint in openSprints() AND assignee = currentUser() ORDER BY updatedDate DESC'
        print("Completed\n=============")
        for item in parse_issues(self.get_response(jql)):
            if item.get('is_completed'):
                print(item.get('statement'))

        print("\nNot Completed\n=============")
        for item in parse_issues(self.get_response(jql)):
            if not item.get('is_completed'):
                print(item.get('statement'))

    def get_backlog_issues(self):
        """Stdout issues of authenticated user whose sprint is empty and unresolved.
        """

        jql = '(Sprint is EMPTY OR Sprint in futureSprints()) \
            AND assignee = currentUser() \
            AND resolution = Unresolved ORDER BY updated DESC'
        print("Backlog\n=============")
        for item in parse_issues(self.get_response(jql)):
            print(item.get('statement'))
        print("\n")



"""Introduction
=============
Jirakosaar is a command line tool to get summary of issues.
Usage
------
python main.py [args]
Example
--------
.. highlight:: bash
.. code-block:: bash
    python main.py backlog
"""

import sys
from os.path import join as join_path
from os import getenv

from jirakosaar.config.configure import Configure
from jirakosaar.issues.main import Issues


# Get home path and set default directory
CONFIG_DIR = join_path(getenv("HOME"), ".jirakosaar")


def run_jirakosaar(argv):
    """Check second index from passed argument and if matches with the
    condition, returns accordingly. If not, return default message and
    exit.
    """
    default = "Avaialble args:\nactive, backlog, configure"

    if not argv:
        print(default)
        sys.exit()

    if argv[0] == 'active':
        issues = Issues(CONFIG_DIR)
        issues.get_current_issues()
        sys.exit()

    if argv[0] == 'backlog':
        issues = Issues(CONFIG_DIR)
        issues.get_backlog_issues()
        sys.exit()

    if argv[0] == 'configure':
        config = Configure(CONFIG_DIR)
        config.set_credential()
        sys.exit()

    print(default)



"""Basic Details of package
"""
from os.path import join

__title__ = 'jirakosaar'
__description__ = 'Command line tool to get summary of jira issues.'
__url__ = 'https://github.com/diwap/jirakosaar'
__version__ = '1.0.4'
__author__ = 'Diwakar Pandey'
__author_email__ = 'dexterisdiwap@gmail.com'
__license__ = 'GPL-30.0 License'
__copyright__ = 'Copyright 2020 Diwakar Pandey'
__scripts__ = [join("bin", "jirakosaar")]



"""configure module create directory in home path and intialize config file
taking input from user. This create a json file and use `json` package to
load and dump file.
"""

import os
import json


# TODO Make this func dynamic to handle different auth and probably logic in auth itself.
def get_input_data(prev_data):
    """Take input from user to set email and api_token in config file.
    """
    try:
        if prev_data:
            return {
                "email": str(input(f"Email({prev_data.get('email')}): ")) \
                    or prev_data.get('email'),
                "api_token": str(input(f"API_TOKEN({prev_data.get('api_token')}): ")) \
                    or prev_data.get('api_token')
            }
        return {
            "email": str(input("Email: ")),
            "api_token": str(input("API_TOKEN: "))
        }
    except KeyboardInterrupt:
        print("\n\nInterrupted! Using previous config if exists")

class Configure:
    """Class needs to import and initialized with config_dir args.
    Basic implementation of Configure class::

        >>> from jirakosaar.config.configure import Configure

        >>> config = Configure("/path/to/file")
        >>> config.set_credential()
    """

    def __init__(self, config_dir: str):
        self.config_dir = config_dir
        self.config_file = os.path.join(self.config_dir, "config.json")

    def get_credential(self):
        """Read config file and read file with readonly permission then
        return data using `json.loads` function.
        """
        try:
            with open(self.config_file, "r") as file:
                return json.loads(file.read())
        except json.decoder.JSONDecodeError:
            pass
        except FileNotFoundError:
            pass

    def set_credential(self):
        """Create dir if doesn't exists. Check for old credential, if
        exists same will be set if input not provided. If input provided,
        old value will be replaced with new one. Any empty input will ask to
        provide input again. Final data is dumped in a json format with
        the indentation of 4.
        """

        try:
            os.mkdir(self.config_dir)
        except FileExistsError:
            pass

        old_cred = self.get_credential()

        with open(self.config_file, "w") as file:
            data = get_input_data(old_cred)

            while not (data.get("email") and data.get("api_token")):
                print("\nNeed both input\n")
                data = get_input_data(old_cred)

            json_obj = json.dumps(data or old_cred, indent=4)

            file.write(json_obj)
            file.write("\n")

            if data:
                print("\nConfiguring credential completed")
